// Copyright (c) 2013, the project authors. Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed 
// by a Apache license that can be found in the LICENSE file.

part of harvet_indexeddb;

class IDBConnection {
  static IDBDatabase _connection;
  
  IDBConnection(this.dbName, this.version): _logger = LoggerFactory.getLoggerFor(IDBConnection);

  Future<IDBCollection> open(String collection) {
    Completer<IDBCollection> completer = new Completer<IDBCollection>();
    
    var request = window.webkitIndexedDB.open(dbName);
    request.on.success.add((connected) {
      _logger.debug("opened DB $dbName");
      IDBDatabase db = connected.target.result;
      if (db.version != version) {
        IDBVersionChangeRequest changeRequest = db.setVersion(version);
        changeRequest.on.success.add((versionChangedEvent) {
          _logger.debug("set DB version $version");
          db.createObjectStore(collection);
          completer.complete(new _IDBCollection(db, collection));
        });
        changeRequest.on.error.add((versionError) => completer.completeError(versionError));
      } else {
        completer.complete(new _IDBCollection(db, collection));
      }
    });
    request.on.error.add((connectionError) => completer.completeError(connectionError));
    
    return completer.future;
  }
  
  final Logger _logger;
  final String dbName;
  final String version;
}

